/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TextualRulePackageVO.as 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.textual
{

import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.AbstractRulePackageVO;
    
/**
* A textual rule package
*
* @author Julien Nicoulaud
*
*/
[RemoteClass(alias="org.ow2.jasmine.rules.manager.model.textual.TextualRulePackage")]
public class TextualRulePackageVO extends AbstractRulePackageVO
                                  implements ITextualRulePackageVO
{
	
	// =========================================================================
	// Variables
	// =========================================================================

    /**
    * The prefix for the package statement line
    */
    public static var PACKAGE_STATEMENT_PREFIX:String = "package ";
    	
	/**
	* The DRL source of the rule package
	*/
	private var _content : String;
    
    
    // =========================================================================
    // Getters and setters
    // =========================================================================
    
    /**
    * Get the content
    */
    [Bindable]
    public function get content():String
    {
        return _content;
    }
        
    /**
    * Set the content
    */    
    public function set content(value:String):void
    {
        // Store the old value
        var oldValue : Object = _content;
        
        // Merge the value with the package statement
        _content = replacePackageName(value,systemName);
        
        // Dispatch a property change event
        notifyPropertyChange("content",oldValue,value);
    }
    
    /**
    * Set the systemName property
    */
    override public function set systemName(value:String):void
    {
        // Assign the value
        super.systemName = value;
        
        // Force refresh the content
        content = replacePackageName(content,value);
    }
    
    /**
    * Get the systemName property
    */
    [Bindable]
    override public function get systemName():String
    {
        return super.systemName;
    }
    
    	
    // =========================================================================
    // Implementation of IRulePackageVO
    // =========================================================================
    
    /**
    * Get the generated source in Drools syntax
    */
    override public function getDrlSource():String
    {
        return content;
    }
    
        
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Replace the package name in the given DRL source with the given package name
    */
    private function replacePackageName(source:String,packageName:String):String
    {
        if (source !== null) {
            var packageStatementLine:RegExp = new RegExp("^package\\s.*","gmi");
            return source.replace(packageStatementLine,PACKAGE_STATEMENT_PREFIX + packageName);
        }
        return null;
    }
}
}
