/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: VisualRulePackageVO.as 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual
{

import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.common.utils.JASMINeLicenseHeaderFactory;
import org.ow2.jasmine.monitoring.eos.common.utils.logger.EoSLogger;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.AbstractRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.textual.ITextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.textual.TextualRulePackageVO;

/**
* A visual rule package
*
* @author Julien Nicoulaud
*/
[RemoteClass(alias="org.ow2.jasmine.rules.manager.model.visual.VisualRulePackage")]
public class VisualRulePackageVO extends AbstractRulePackageVO
                                 implements IVisualRulePackageVO
{
	
	// =========================================================================
	// Variables
	// =========================================================================
    
    /**
    * The rules
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.IRuleVO')]
    private var _rules : ArrayCollection;
    
    /**
    * The imports
    */
    [ArrayElementType('String')]
    private var _imports : ArrayCollection;
	
    
    // =========================================================================
    // Getters and setters
    // =========================================================================
    
    /**
    * Set the rules collection
    */    
    public function set rules(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _rules;
        
        // Assign the value
        _rules = value;
        
        // Bind some properties for each rule
        for each (var rule :IRuleVO in rules) {
            rule.imports = imports;
        }
                
        // Listen to the rules collection changes
        _rules.addEventListener(CollectionEvent.COLLECTION_CHANGE,rulesListChangeHandler,false,0,true);
        
        // Dispatch a property change event
        notifyPropertyChange("rules",oldValue,value);
    }
    
    /**
    * Get the rules collection
    */
    [Bindable]
    public function get rules():ArrayCollection
    {
        // If the field is not initialized, do it
        if (_rules == null) {
            _rules = new ArrayCollection();
            _rules.addEventListener(CollectionEvent.COLLECTION_CHANGE,rulesListChangeHandler,false,0,true);
        }
        return _rules;
    }
    
    /**
    * Set the imports collection
    */    
    public function set imports(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _imports;
        
        // Assign the value
        _imports = value;
        
        // Bind some properties for each rule
        for each (var rule :IRuleVO in rules) {
            rule.imports = imports;
        }

        // Dispatch a property change event
        notifyPropertyChange("imports",oldValue,value);
    }
    
    /**
    * Get the imports collection
    */
    [Bindable]
    public function get imports():ArrayCollection
    {
        // Initialize the property if needed
        if (_imports == null) {
            _imports = new ArrayCollection();
        }
        return _imports;
    }
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Get the generated source in Drools syntax
    */
    override public function getDrlSource():String
    {
    	var source : String = "";
    	
    	// Format the package statement
    	source += "package " + systemName + "\n";
    	
    	var rule:IRuleVO;
    	
    	// Format the imports section
        for each (var importStatement : String in imports) {
            source += "import " + importStatement + ";\n";
        }
    	
    	// Format rules
    	for each (rule in rules) {
    		source += "\n\n" + rule.getDrlSource();
    	}
    	
        
        return source + "\n";
    }
    
    /**
    * Generate a textual rule package from the visual rule package
    */
    public function getTextualPackage():ITextualRulePackageVO
    {
        var res : ITextualRulePackageVO = new TextualRulePackageVO();
        res.description = description;
        res.ruleEngineInstanceName = ruleEngineInstanceName;
        res.previousSystemName = previousSystemName;
        res.content = JASMINeLicenseHeaderFactory.getDRLHeader()
                      + "\n" + this.getDrlSource();
        res.displayName = displayName;
        res.state = state;
        return res;
    }
    
    
    // =========================================================================
    // Events handlers
    // =========================================================================
    
    /**
    * When the rules list changes
    */
    private function rulesListChangeHandler(event:CollectionEvent):void
    {
        // Bind some synchronized variables
        switch (event.kind) {
            case CollectionEventKind.ADD:
                for each (var rule : IRuleVO in event.items) {
                    rule.imports = imports;
                }
                break;
        }
        
        // Notify the change
        notifyPropertyChange("rules",_rules,rules);
    }
    
}
}
